import 'dart:io';

import 'package:tingyun_flutter_plugin/src/agent/util/logger.dart';

class TingyunHeaders extends HttpHeaders {
  HttpHeaders _httpHeaders;
  Map<String, String> addExtraHeader;
  Map<String, String> apmsInfosHeaders;

  ///  这里用来判断TingyunHttpClientRequest构造函数是否完成，因为在那个构造函数里面
  ///  添加跨应用，等头到request时，都会走下面的add函数的接口
  bool isHttpRequestInited = false;

  TingyunHeaders(this._httpHeaders, this.apmsInfosHeaders)
      : addExtraHeader = {};

  @override
  List<String>? operator [](String name) {
    return this._httpHeaders[name];
  }

  Map<String, String> get addExtraHeadersFun => this.addExtraHeader;

  @override
  void add(String name, Object value, {bool preserveHeaderCase = false}) {
    Logger().d('add name:' +
        name +
        ", value:" +
        value.toString() +
        ", apmInfoHeaders:" +
        this.apmsInfosHeaders.toString());
    // //如果apms存在的话，此时应该以客户添加的add为主
    // if (isHttpRequestInited && this.apmsInfosHeaders.containsKey(name)) {
    //   this._httpHeaders.removeAll(name);
    //   //存在的话就记录下来
    //   if (value is String) {
    //     Logger().d('addExtraHeader:' + addExtraHeader.toString());
    //     addExtraHeader.putIfAbsent(name, () => value);
    //   }
    // }
    processApmInfo(name, value);
    this._httpHeaders.add(name, value, preserveHeaderCase: preserveHeaderCase);
  }

  void processApmInfo(String name, Object value){
    //如果apms存在的话，此时应该以客户添加的add为主
    if (isHttpRequestInited && this.apmsInfosHeaders.containsKey(name.toLowerCase())) {
      this._httpHeaders.removeAll(name);
      //存在的话就记录下来
      if (value is String) {
        Logger().d('addExtraHeader:' + addExtraHeader.toString());
        addExtraHeader.putIfAbsent(name.toLowerCase(), () => value);
      }
    }
  }

  @override
  void clear() {
    this._httpHeaders.clear();
  }

  @override
  void forEach(void Function(String name, List<String> values) action) {
    // _headers.forEach((String name, List<String> values) {
    //   String originalName = _originalHeaderName(name);
    //   action(originalName, values);
    // });
    this._httpHeaders.forEach(action);
  }

  @override
  void noFolding(String name) {
    this._httpHeaders.noFolding(name);
  }

  @override
  void remove(String name, Object value) {
    this._httpHeaders.remove(name, value);
  }

  @override
  void removeAll(String name) {
    this._httpHeaders.removeAll(name);
  }

  @override
  void set(String name, Object value, {bool preserveHeaderCase = false}) {
    Logger().d('set name:' + name + ", value:" + value.toString() +
        ", apmInfoHeaders:" +
        this.apmsInfosHeaders.toString());
    processApmInfo(name, value);

    this._httpHeaders.set(name, value, preserveHeaderCase: preserveHeaderCase);
  }

  @override
  String? value(String name) {
    this._httpHeaders.value(name);
  }
}
