import 'dart:convert';
import 'dart:io';

import 'package:tingyun_flutter_plugin/src/agent/agent_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/http/dns_info.dart';
import 'package:tingyun_flutter_plugin/src/agent/http/http_header.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/logger.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/string_utils.dart';

import '../../../tingyun_flutter_plugin.dart';

class TingyunHttpClientRequest implements HttpClientRequest {

  HttpClientRequest _httpClientRequest;
  TingyunFlutterImpl? _tingyun;
  // int _dnsDuration;
  // String _ipInfo;
  /// 用于标识一个网络，因为我们的跨应用是个固定值，所以这里弄了个uuid
  String _uuid;
  List<String> retriveHeaderValue;
  Map<String, String> apmNotExistHeader;
  Map<String, String> apmExistHeader;
  Map<String, String> allApmInfos;

  TingyunHeaders _tingyunHeaders;

  static final String _tingyunHeader = "X-Tingyun";
  //response X-Tingyun-Data

  TingyunHttpClientRequest(this._httpClientRequest, this._tingyun, Map<String, String> _headerValue, this._uuid,
      this.retriveHeaderValue, this.allApmInfos): this.apmNotExistHeader = {},
        this.apmExistHeader = {}, _tingyunHeaders = TingyunHeaders(_httpClientRequest.headers, allApmInfos)
        {
    _addTingyunHeader(_headerValue);
    _addApmsHeader(allApmInfos);
    Logger().d('TingyunHttpClientRequest retriveHeaderValue:' +
        this.retriveHeaderValue.toString() + "，apmHeaders:" + allApmInfos.toString());
    DnsInfo.setDnsInfo(uri.host);
    _tingyunHeaders.isHttpRequestInited = true;
  }

  /// 这里我们是否需要添加我们的header来实现跨应用追踪？
  void _addTingyunHeader(Map<String, String> headerValue) {
    if(headerValue.length > 0){
      headerValue.forEach((key, value) {
        if(this.headers.value(key) == null) {
            this.headers.add(key, value);
            Logger().d("add header: key:" + key + ", value:" + value);
        }else{
          Logger().d("not add header becaute header is exist, key:" + key);
        }
      });
    }
  }

  void _addApmsHeader(Map<String, String> headerValue){
    if(headerValue.length > 0){
      headerValue.forEach((key, value) {
        if(this.headers.value(key) == null) {
          this.headers.add(key, value);
          // this.apmNotExistHeader.putIfAbsent(key, () => value);
          Logger().d("add apms header: key:" + key + ", value:" + value);
        }else{
            // this.apmExistHeader.putIfAbsent(key, () => this.headers.value(key)!);
          Logger().d("not add apm header becaute header is exist, key:" + key + ", value:" + value);
        }
      });
    }
  }

  @override
  void abort([Object? exception, StackTrace? stackTrace]) {
    _httpClientRequest.addError(exception!, stackTrace);
  }

  @override
  bool get bufferOutput => _httpClientRequest.bufferOutput;

  @override
  set bufferOutput(bool _bufferOutput)  => _httpClientRequest.bufferOutput = _bufferOutput;

  @override
  int get contentLength => _httpClientRequest.contentLength;

  @override
  set contentLength(int _contentLength) =>_httpClientRequest.contentLength = _contentLength;

  @override
  Encoding get encoding => _httpClientRequest.encoding;

  @override
  set encoding(Encoding value) => _httpClientRequest.encoding = value;

  @override
  bool get followRedirects => _httpClientRequest.followRedirects;

  @override
  set followRedirects(bool value) => _httpClientRequest.followRedirects = value;

  @override
  int get maxRedirects => _httpClientRequest.maxRedirects;

  @override
  set maxRedirects(int value) => _httpClientRequest.maxRedirects = value;

  @override
  bool get persistentConnection => _httpClientRequest.persistentConnection;

  @override
  set persistentConnection(bool value) =>
      _httpClientRequest.persistentConnection = value;

  @override
  void add(List<int> data) {
    _httpClientRequest.add(data);
  }

  @override
  void addError(Object error, [StackTrace? stackTrace]) {
    _httpClientRequest.addError(error, stackTrace);
  }

  @override
  Future addStream(Stream<List<int>> stream) {
    return _httpClientRequest.addStream(stream);
  }

  void _processApmsNotExist(){
    this.apmExistHeader.addAll(_tingyunHeaders.addExtraHeadersFun);
    this.allApmInfos.forEach((key, value) {
      Logger().d("apmExistHeader:" + this.apmExistHeader.toString());
      if(!this.apmExistHeader.containsKey(key)){
        this.apmNotExistHeader.putIfAbsent(key, () => value);
      }
    });
  }

  @override
  Future<HttpClientResponse> close() async{
    //无论是否是跨应用的我们的header的，我们都监控？
    // if (this.headers.value(_tingyunHeader) != null) {
      TYWebRequestTiming timing = _tingyun!.createWebRequestTiming(
          _uuid, _httpClientRequest.uri.toString());

      timing.start();
      try {
        HttpClientResponse response = await _httpClientRequest.close();
        //结束的时候处理一下apms
        _processApmsNotExist();

        // int dnsDura = DnsInfo.getDnsDuration(uri.host);
        // String ip = DnsInfo.getIpAddress(uri.host);
        // timing.withDnsInfo(dnsDura, ip);
        // timing.stopWebRequestTiming(response.statusCode, response.reasonPhrase);
        Map<String, String> responseHeadInfo = {};
        for(String info in retriveHeaderValue){
          Logger().d("retriveHeaderValue header: key:" + info);
          if(response.headers.value(info) != null){
            responseHeadInfo.putIfAbsent(info, () => response.headers.value(info)!);
          }
        }
        Logger().d("statusCode:" + response.statusCode.toString()
         + ", responseHeadInfo:" + responseHeadInfo.toString()
            + ", apmExistHeader:" + apmExistHeader.toString()
            + ", apmNotExistHeader:" + apmNotExistHeader.toString()
        );
        timing.stop(response.statusCode, retriveResponseHeader: responseHeadInfo, apmExistInfo: apmExistHeader, apmNotExistHeader: apmNotExistHeader);

        return response;
      } catch (error) {
        // timing.stopWebRequestTiming(-1, error.toString());
        timing.stop(-1);

        throw error;
      }
    // } else {
    //   return await _httpClientRequest.close();
    // }
  }

  @override
  HttpConnectionInfo? get connectionInfo => _httpClientRequest.connectionInfo;

  @override
  List<Cookie> get cookies => _httpClientRequest.cookies;

  @override
  Future<HttpClientResponse> get done => _httpClientRequest.done;

  @override
  Future flush() {
    return _httpClientRequest.flush();
  }

  @override
  HttpHeaders get headers {
    // return TingyunHeaders(_httpClientRequest.headers, apmExistHeader, allApmInfos);
    return _tingyunHeaders;
  }
  // HttpHeaders get headers => _httpClientRequest.headers;

  @override
  String get method => _httpClientRequest.method;

  @override
  Uri get uri => _httpClientRequest.uri;

  @override
  void write(Object? obj) {
    _httpClientRequest.write(obj);
  }

  @override
  void writeAll(Iterable objects, [String separator = ""]) {
    _httpClientRequest.writeAll(objects, separator);
  }

  @override
  void writeCharCode(int charCode) {
    _httpClientRequest.writeCharCode(charCode);
  }

  @override
  void writeln([Object? obj = ""]) {
    _httpClientRequest.writeln(obj);
  }
}